@@ -164,6 +164,7 @@ class Update(TestCase):
 
     # TODO: test update with external urlid which doesn't exist
     # TODO: test update with internal urlid which doesn't exist
+    # TODO: repeat of the above where the relationship is ForeignKey
     # TODO: test update with internal urlid which refers to a different type of object entirely
     # TODO: test update with internal urlid which refers to a container
 
@@ -388,7 +389,7 @@ class Update(TestCase):
                 {
                     '@id': "{}/resources/{}/joboffers/".format(settings.BASE_URL, resource.pk),
                     'ldp:contains': [
-                        {'@id': 'http://testserver.com/job-offers/{}/'.format(job.slug),
+                        {'@id': job.urlid,
                          'http://happy-dev.fr/owl/#title': "new job",
                          },
                     ]
@@ -399,8 +400,7 @@ class Update(TestCase):
                                    data=json.dumps(body),
                                    content_type='application/ld+json')
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.data['joboffers']['ldp:contains'][0]['@id'],
-                         "http://testserver.com/job-offers/{}/".format(job.slug))
+        self.assertEqual(response.data['joboffers']['ldp:contains'][0]['@id'], job.urlid)
         self.assertEqual(response.data['joboffers']['ldp:contains'][0]['title'], "new job")
 
     def test_m2m_new_link_embedded(self):
@@ -430,7 +430,7 @@ class Update(TestCase):
                 # '@id': "http://testserver/resources/{}/joboffers/".format(resource.pk),
                 'ldp:contains': [
                     {
-                        '@id': 'http://testserver.com/job-offers/{}/'.format(job.slug),
+                        '@id': job.urlid,
                         'http://happy-dev.fr/owl/#title': "new job",
                     }
                 ]
@@ -441,11 +441,10 @@ class Update(TestCase):
                                    data=json.dumps(body),
                                    content_type='application/ld+json')
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.data['joboffers']['ldp:contains'][0]['@id'],
-                         "http://testserver.com/job-offers/{}/".format(job.slug))
+        self.assertEqual(response.data['joboffers']['ldp:contains'][0]['@id'], job.urlid)
         self.assertEqual(response.data['joboffers']['ldp:contains'][0]['title'], "new job")
 
-    def test_m2m_new_link_federated(self):
+    def test_m2m_new_link_external(self):
         resource = Resource.objects.create()
         body = {
             'http://happy-dev.fr/owl/#joboffers': {
@@ -459,21 +458,8 @@ class Update(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.data['joboffers']['ldp:contains'][0]['@id'],
                          "http://external.job/job/1")
-
-    def test_m2m_new_link_external(self):
-        resource = Resource.objects.create()
-        body = {
-            'http://happy-dev.fr/owl/#joboffers': {
-                '@id': 'http://testserver.com/job-offers/stuff/',
-            }
-        }
-
-        response = self.client.put('/resources/{}/'.format(resource.pk),
-                                   data=json.dumps(body),
-                                   content_type='application/ld+json')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.data['joboffers']['ldp:contains'][0]['@id'],
-                         "http://testserver.com/job-offers/stuff/")
+        self.assertIn('@type', response.data['joboffers']['ldp:contains'][0])
+        self.assertEqual(len(response.data['joboffers']['ldp:contains'][0].items()), 2)
 
     def test_m2m_new_link_local(self):
         resource = Resource.objects.create()
@@ -530,6 +516,8 @@ class Update(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.data['team']['ldp:contains'][0]['@id'],
                          "http://external.user/user/1")
+        self.assertIn('@type', response.data['team']['ldp:contains'][0])
+        self.assertEqual(len(response.data['team']['ldp:contains'][0].items()), 2)
 
     def test_m2m_user_link_existing_external(self):
         project = Project.objects.create(description="project name")
@@ -546,6 +534,8 @@ class Update(TestCase):
                                    content_type='application/ld+json')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.data['team']['ldp:contains'][0]['@id'], ext_user.urlid)
+        self.assertIn('@type', response.data['team']['ldp:contains'][0])
+        self.assertEqual(len(response.data['team']['ldp:contains'][0].items()), 2)
 
         project = Project.objects.get(pk=project.pk)
         self.assertEqual(project.team.count(), 1)
@@ -611,7 +601,7 @@ class Update(TestCase):
             "last_name": "Bourlier",
             "username": "alex",
             'userprofile': {
-                '@id': "http://happy-dev.fr/userprofiles/{}/".format(profile.pk),
+                '@id': profile.urlid,
                 'description': "user update"
             },
             '@context': {
@@ -628,4 +618,7 @@ class Update(TestCase):
                                    content_type='application/ld+json')
         self.assertEqual(response.data['description'], "user update")
 
+    # TODO: test passing foreign key relation which I shouldn't have access/permission to
+    # TODO: test passing many-to-many relation in edit which isn't yet on my model
+
 
